# Standard library imports:
import logging
import os

# P3 imports:
from p3.base.exception import BaseModuleError
from p3.base.path_helpers import full_path_split


_LOG = logging.getLogger("p3.base")


## Make release package list as expected by distutils.core.setup function's
#  packages parameter.
def make_package_list(root_path):
    packages = []

    if not os.path.isdir(root_path):
        raise BaseModuleError(
            BaseModuleError.codes.invalid_directory_path,
            "The '{0:s}' path doesn't exist or is not a directory.".format(root_path))

    norm_path = os.path.normpath(root_path)

    if norm_path <> root_path:
        _LOG.info(
            "Normalized given root path: '{0:s}' -> '{1:s}'.".format(root_path, norm_path))

    norm_parts = full_path_split(norm_path)
    norm_parts_cnt = len(norm_parts)

    for dir_path, subdir_names, file_names in os.walk(norm_path, followlinks=True):
        dir_parts = full_path_split(dir_path)[norm_parts_cnt:]

        if '__init__.py' in file_names:
            packages.append('.'.join(dir_parts))

    return packages


def make_short_version_str(version_info):
    vi = version_info
    if vi.releaselevel == 'final':
        if vi.serial:
            return "{0:d}.{1:d}.{2:d}-{3:d}".format(
                vi.major, vi.minor, vi.micro, vi.serial)
        else:
            return "{0:d}.{1:d}.{2:d}".format(
                vi.major, vi.minor, vi.micro)
    else:
        level = { 'alpha': 'a', 'beta': 'b', 'rc': 'c' }.get(
            vi.releaselevel)

        assert level is not None

        if vi.serial:
            return "{0:d}.{1:d}.{2:d}-{3:s}{4:d}".format(
                vi.major, vi.minor, vi.micro, level, vi.serial)
        else:
            return "{0:d}.{1:d}.{2:d}-{3:s}".format(
                vi.major, vi.minor, vi.micro, level)
