#
#  FILE NAME:
#      index_transform.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import logging

# P3 imports:
from p3.data.error import DataModuleError


_LOG = logging.getLogger( "p3.data" )


def index_transform( src_table, dst_table, index_cols, transform_cb ):
    src_col_idx = [c.index for c in src_table.resolve_columns( *index_cols )]
    row_lut = {}

    def make_key( row, col_idx ):
        return tuple(row[i] for i in col_idx)

    for src_row in src_table.rows:
        dst_row = row_lut.get( make_key(src_row, src_col_idx) )
        new_row = transform_cb( src_row, dst_row, dst_table )

        if dst_row is None:
            row_lut[make_key(src_row, src_col_idx)] = new_row
