#
#  FILE NAME:
#      enum_converter.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import logging
import numbers
import re

# P3 imports:
from p3.base import Enum
from p3.logic.var.converter.var_converter import VarConverter


_LOG = logging.getLogger( "p3.logic" )


class EnumConverter( VarConverter ):
    errors = Enum( "format_error", "empty_input", "value_error" )

    ## ===[ Initialization ]==================================================================== ##

    def __init__( self, enum ):
        super( EnumConverter, self ).__init__(
            self._deserialize, None, lambda v: isinstance( v, numbers.Integral ) )

        self.enum = enum


    ## ===[ Conversion ]======================================================================== ##

    def _deserialize( self, res ):
        if not res.in_val:
            res.errors.append( self.errors.empty_input )
            return None

        m = re.match( "^-{0,1}[1-9]{1}[0-9]*$|^0$", res.in_val )

        if m is not None:
            out_val = int(m.group(0))

            if out_val in self.enum:
                return out_val
            else:
                res.errors.append( self.errors.value_error )
                return None
        else:
            res.errors.append( self.errors.format_error )
            return None
