from p3.base import challenge_module

# Standard library imports:
import Cookie
import logging

# Third party imports:
challenge_module('django')
import django.http
import django.template.loader

# P3 imports:
from p3.base import import_object, Enum
from p3.logic.response import JsonResponse, PageResponse, RedirectResponse


_LOG = logging.getLogger("p3.web")


## Convert cookies stored in logic_response object (p3.logic.response.HttpResponse like) and add it
#   to the django_response object.
def django_transfer_cookies(django_response, logic_response):
    ## @note morsel is expected to be Cookie.Morsel like object.
    for morsel in logic_response.cookies:
        # The unset attributes must be adapted as the HttpResponse.set_cookie method expects None
        # for unspecified max_age, expires, domain and secure attributes and the Cookie.Morsel uses
        # empty string for them.
        # See source of Cookie.Morsel.__init__ and django.http.HttpResponse.set_cookie methods.
        attr_in = (
            ('max_age', morsel['max-age']),
            ('expires', morsel['expires']),
            ('path',    morsel['path']),
            ('domain',  morsel['domain']),
            ('secure',  morsel['secure']))
        attributes = dict((k, v) for k, v in attr_in if v)
        django_response.set_cookie(morsel.key, morsel.value, **attributes)


def _render_django_default(response):
    return django.template.loader.render_to_string(
        response.template_name, response.template_context )


def serve_django_page(request, logic, render_cb=_render_django_default):
    _LOG.info("Serving page generated by '{0:s}' class".format(logic))
    
    cls = import_object(logic)
    obj = cls(request)
    obj.execute()

    if isinstance(obj.response, PageResponse):
        rendered = render_cb(obj.response)
        response = django.http.HttpResponse(content=rendered, status=200, content_type='text/html')
    elif isinstance(obj.response, RedirectResponse):
        response = django.http.HttpResponseRedirect(obj.response.url)
    else:
        assert False, "Unsupported response type."

    django_transfer_cookies(response, obj.response)

    return response


## Convert p3 FileResponse object to django HttpResponse object.
#  @param response FileReponse like object.
def django_adapt_file_response(response):
    dr = django.http.HttpResponse(
        content=response.content, mimetype=response.mime_type )
    if response.file_name is not None:
        dr['Content-Disposition'] = (
            "attachment; filename={0:s}".format(response.file_name) )
    return dr
